@@ -30,8 +30,8 @@ from couchdbkit.schema.base import DocumentSchema, ALLOWED_PROPERTY_TYPES
 __all__ = ['SchemaProperty']
 
 class SchemaProperty(Property):
-    """ Schema property. It allow you add a DocumentSchema instance 
-    a member of a Document object. It return a
+    """ Schema property. It allows you add a DocumentSchema instance 
+    a member of a Document object. It returns a
    `schemaDocumentSchema` object.
 
     Exemple :
@@ -130,4 +130,4 @@ class SchemaProperty(Property):
             if not isinstance(value, dict):
                 raise BadValueError("%s is not a dict" % str(value))
             value = schema(**value)
-        return value._doc
\ No newline at end of file
+        return value._doc
