@@ -968,6 +968,11 @@ class PropertyTestCase(unittest.TestCase):
         b.ls.append(datetime.utcnow())
         self.assertRaises(BadValueError, b.validate)
         
+        b1  = B()
+        b1.ls = [u'hello', u'123']
+        self.assert_(b1.ls == [u'hello', u'123'])
+        self.assert_(b1._doc['ls'] == [u'hello', u'123'])
+
          
     def testDictProperty(self):
         from datetime import datetime
